const chalk = require("chalk");

module.exports = function enforceDescriptiveVariables(ast, filename) {
  let issues = [];

  require("@babel/traverse").default(ast, {
    VariableDeclarator(path) {
      const id = path.node.id;
      if (id && id.type === "Identifier" && typeof id.name === "string") {
        const name = id.name;

        // Rule 1: Check if the variable name is too short (less than 3 characters)
        if (name.length < 3) {
          issues.push({
            line: path.node.loc.start.line,
            message: `Variable "${name}" should have a descriptive name.`,
            suggestion: `The variable name "${name}" is too short. Try using a more descriptive name like "userAge", "itemCount", or "orderAmount" instead.`,
          });
        }

        // Rule 2: Check if the variable name is not all lowercase (inconsistent naming convention)
        else if (/^[a-z]+$/.test(name) === false) {
          // If it's a component name (e.g., UserAge), it should follow PascalCase
          if (/^[A-Z]/.test(name)) {
            issues.push({
              line: path.node.loc.start.line,
              message: `Component name "${name}" should follow PascalCase convention.`,
              suggestion: `The component name "${name}" should start with an uppercase letter and follow PascalCase convention. For example, "UserProfile" or "ProductList".`,
            });
          } else {
            issues.push({
              line: path.node.loc.start.line,
              message: `Variable "${name}" should have a descriptive name.`,
              suggestion: `The variable name "${name}" contains uppercase letters or symbols. Please use lowercase letters only, like "userAge" or "itemCount".`,
            });
          }
        }

        // Rule 3: Check for unclear or non-descriptive names (like 'a', 'b', etc.)
        else if (/^[a-z]{1,2}$/.test(name)) {
          issues.push({
            line: path.node.loc.start.line,
            message: `Variable "${name}" should have a descriptive name.`,
            suggestion: `The variable name "${name}" is too vague. Please use a more descriptive name that explains its purpose, such as "userAge", "itemCount", or "orderTotal".`,
          });
        }

        // Rule 4: Check for variables that should use camelCase (common practice in JavaScript)
        else if (/[^a-zA-Z0-9]/.test(name)) {
          issues.push({
            line: path.node.loc.start.line,
            message: `Variable "${name}" should have a descriptive name.`,
            suggestion: `The variable name "${name}" contains invalid characters. Please use camelCase for naming variables, such as "userAge" or "orderAmount".`,
          });
        }
      }
    },

    // Check for event handler functions
    FunctionDeclaration(path) {
      const { id } = path.node;
      if (id && typeof id.name === "string" && /^(handle|on)[A-Z]/.test(id.name) === false) {
        issues.push({
          line: path.node.loc.start.line,
          message: `Event handler function "${id.name}" should follow the naming convention.`,
          suggestion: `Event handler functions should start with "handle" or "on", like "handleClick" or "onSubmit". For example, rename "${id.name}" to "handleClick" or "onItemSelect".`,
        });
      }
    },

    // Check for component names
    JSXElement(path) {
      const name = path.node.openingElement.name.name;
      // Only check for custom component names (that start with an uppercase letter)
      if (/^[A-Z]/.test(name) && !/^[a-z]/.test(name)) {
        issues.push({
          line: path.node.loc.start.line,
          message: `Component name "${name}" should follow PascalCase convention.`,
          suggestion: `The component name "${name}" should start with an uppercase letter and follow PascalCase convention. For example, "UserProfile" or "ProductList".`,
        });
      }
    },
  });

  if (issues.length > 0) {
    console.log(chalk.red(`\nIssues in ${filename}:`));
    issues.forEach((issue) => {
      console.log(chalk.yellow(`Line ${issue.line}: ${issue.message}`));
      console.log(chalk.green(`Suggestion: ${issue.suggestion}`));
    });
  }
};
